@@ -23,6 +23,7 @@ from diskimage_builder.block_device.plugin import PluginBase
 from diskimage_builder.block_device.utils import exec_sudo
 from diskimage_builder.block_device.utils import parse_abs_size_spec
 from diskimage_builder.block_device.utils import parse_rel_size_spec
+from diskimage_builder.block_device.utils import remove_device
 
 
 logger = logging.getLogger(__name__)
@@ -94,6 +95,8 @@ class Partitioning(PluginBase):
 
     def _create_mbr(self):
         """Create partitions with MBR"""
+        # NOTE(TheJulia): This is funcitonally incompatible with block/sector
+        # sizing other than 512 bytes.
         with MBR(self.image_path, self.disk_size, self.align) as part_impl:
             for part_cfg in self.partitions:
                 part_name = part_cfg.get_name()
@@ -117,15 +120,19 @@ class Partitioning(PluginBase):
                 # below once we're done.  So the device this partition
                 # will be seen at becomes "/dev/mapper/loop0pX"
                 assert self.device_path[:5] == "/dev/"
-                partition_device_name = "/dev/mapper/%sp%d" % \
-                                        (self.device_path[5:], part_no)
+                device_name = "%sp%d" % (self.device_path[5:], part_no)
+                device_path = "/dev/mapper/%s" % device_name
                 self.state['blockdev'][part_name] \
-                    = {'device': partition_device_name}
+                    = {'device': device_path}
+                part_cfg.add_rollback(remove_device, device_name)
 
     def _create_gpt(self):
         """Create partitions with GPT"""
 
-        cmd = ['sgdisk', self.image_path]
+        # Use the loopback via device_path, as using the file means the
+        # partitioning is exposed to sector sizing of the OS, not of the
+        # underlying "device" provided by the loopback.
+        cmd = ['sgdisk', self.device_path]
 
         # This padding gives us a little room for rounding so we don't
         # go over the end of the disk
@@ -145,6 +152,9 @@ class Partitioning(PluginBase):
             # megabytes so it can align on sensible boundaries. And
             # create partitions right after previous so no need to
             # calculate start/end - just size.
+            if not size <= disk_free:
+                logger.error('The requested size of the image is smaller '
+                             'than the block device configuration.')
             assert size <= disk_free
             args['size'] = size // (1024 * 1024)
 
@@ -158,14 +168,16 @@ class Partitioning(PluginBase):
             # below once we're done.  So the device this partition
             # will be seen at becomes "/dev/mapper/loop0pX"
             assert self.device_path[:5] == "/dev/"
-            device_name = "/dev/mapper/%sp%d" % (self.device_path[5:], pnum)
+            device_name = "%sp%d" % (self.device_path[5:], pnum)
+            device_path = "/dev/mapper/%s" % device_name
             self.state['blockdev'][p.get_name()] \
-                = {'device': device_name}
+                = {'device': device_path}
 
             disk_free = disk_free - size
             pnum = pnum + 1
             logger.debug("Partition %s added, %s remaining in disk",
                          pnum, disk_free)
+            p.add_rollback(remove_device, device_name)
 
         logger.debug("cmd: %s", ' '.join(cmd))
         exec_sudo(cmd)
